@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2017 sqlmap developers (http://sqlmap.org/)
-See the file 'doc/COPYING' for copying permission
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
+See the file 'LICENSE' for copying permission
 """
 
 import threading
@@ -11,7 +11,7 @@ from lib.core.data import logger
 from lib.core.enums import CUSTOM_LOGGING
 from lib.core.enums import TIMEOUT_STATE
 
-def timeout(func, args=(), kwargs={}, duration=1, default=None):
+def timeout(func, args=None, kwargs=None, duration=1, default=None):
     class InterruptableThread(threading.Thread):
         def __init__(self):
             threading.Thread.__init__(self)
@@ -20,10 +20,10 @@ def timeout(func, args=(), kwargs={}, duration=1, default=None):
 
         def run(self):
             try:
-                self.result = func(*args, **kwargs)
+                self.result = func(*(args or ()), **(kwargs or {}))
                 self.timeout_state = TIMEOUT_STATE.NORMAL
-            except Exception, msg:
-                logger.log(CUSTOM_LOGGING.TRAFFIC_IN, msg)
+            except Exception as ex:
+                logger.log(CUSTOM_LOGGING.TRAFFIC_IN, ex)
                 self.result = default
                 self.timeout_state = TIMEOUT_STATE.EXCEPTION
 
@@ -31,7 +31,7 @@ def timeout(func, args=(), kwargs={}, duration=1, default=None):
     thread.start()
     thread.join(duration)
 
-    if thread.isAlive():
+    if thread.is_alive():
         return default, TIMEOUT_STATE.TIMEOUT
     else:
         return thread.result, thread.timeout_state
